/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.jcg;

import java.util.Arrays;
import javax.sql.DataSource;
import org.springframework.batch.core.Job;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.JobParameters;
import org.springframework.batch.core.JobParametersBuilder;
import org.springframework.batch.core.Step;
import org.springframework.batch.core.job.builder.JobBuilder;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.batch.core.repository.JobRepository;
import org.springframework.batch.core.step.builder.StepBuilder;
import org.springframework.batch.item.ItemWriter;
import org.springframework.batch.item.database.JdbcCursorItemReader;
import org.springframework.batch.item.database.builder.JdbcCursorItemReaderBuilder;
import org.springframework.batch.item.file.FlatFileItemReader;
import org.springframework.batch.item.file.builder.FlatFileItemReaderBuilder;
import org.springframework.batch.item.support.CompositeItemReader;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;
import org.springframework.transaction.PlatformTransactionManager;

/**
 *
 * @author omozegieaziegbe
 */
@Configuration
//@EnableBatchProcessing
public class BatchConfig {

    @Bean
    public DataSource dataSource() {
        return DataSourceBuilder.create()
                .driverClassName("org.h2.Driver")
                .url("jdbc:h2:mem:batchdb;DB_CLOSE_DELAY=-1;")
                .username("sa")
                .password("")
                .build();
    }

    @Bean
    public FlatFileItemReader<Customer> fileReader() {
        return new FlatFileItemReaderBuilder<Customer>()
                .name("customerFileItemReader")
                .resource(new ClassPathResource("customers.csv"))
                .delimited()
                .names("id", "name", "email")
                .linesToSkip(1)
                .targetType(Customer.class)
                .build();
    }

    @Bean
    public JdbcCursorItemReader<Customer> databaseCustomerReader() {
        String sql = "select * from customers";
        return new JdbcCursorItemReaderBuilder<Customer>()
                .name("customerTableItemReader")
                .dataSource(dataSource())
                .sql(sql)
                .beanRowMapper(Customer.class)
                .build();
    }

    @Bean
    public ItemWriter<Customer> customerWriter() {
        return items -> {
            for (Customer customer : items) {
//                System.out.println("Writing customer: " + "ID: " + customer.getId()
//                        + ", Name: " + customer.getName()
//                        + ", Email: " + customer.getEmail());
            }
        };
    }

    @Bean
    public CompositeItemReader<Customer> itemReader() {
        return new CompositeItemReader<>(Arrays.asList(fileReader(), databaseCustomerReader()));
    }

    @Bean
    public Step customerStep(JobRepository jobRepository, PlatformTransactionManager transactionManager) {
        return new StepBuilder("customerStep", jobRepository)
                .<Customer, Customer>chunk(10, transactionManager)
                .reader(itemReader())
                .writer(customerWriter())
                .build();
    }


    @Bean
    public Job customerJob(JobRepository jobRepository, Step customerStep) {
        return new JobBuilder("customerJob", jobRepository)
                .start(customerStep)
                .build();
    }

    @Bean
    public CommandLineRunner run(JobLauncher jobLauncher, Job customerJob) {
        return args -> {
            JobParameters jobParameters = new JobParametersBuilder()
                    .addLong("startTime", System.currentTimeMillis())
                    .toJobParameters();

            JobExecution jobExecution = jobLauncher.run(customerJob, jobParameters);
            System.out.println("Job Status: " + jobExecution.getStatus());
        };
    }
}
