/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.jcg;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.JobParameters;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.item.support.CompositeItemReader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.test.context.ContextConfiguration;

/**
 *
 * @author omozegieaziegbe
 */
@SpringBootTest
@EnableAutoConfiguration
@ContextConfiguration(classes = {BatchConfig.class})
public class CompositeItemReaderTest {

    @Autowired
    @Qualifier("itemReader")
    private CompositeItemReader<Customer> compositeReader;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @BeforeEach
    public void setUp() {
        jdbcTemplate.update("DELETE FROM customers");
        jdbcTemplate.update("INSERT INTO customers (id, name, email) VALUES (?, ?, ?)",
                102, "Frank Riberry", "f.riberry@jcg.com");
    }

    @Test
    public void testCompositeItemReader() throws Exception {
        StepExecution stepExecution = new StepExecution("customerStep", new JobExecution(1L, new JobParameters()), 1L);
        ExecutionContext executionContext = stepExecution.getExecutionContext();
        compositeReader.open(executionContext);

        // Read first customer from CSV
        Customer customer1 = compositeReader.read();
        assertNotNull(customer1);
        assertEquals(1, customer1.getId());  // From CSV
        assertEquals("John Fish", customer1.getName());

        // Read second customer from CSV
        Customer customer2 = compositeReader.read();
        assertNotNull(customer2);
        assertEquals(2, customer2.getId());  // From CSV
        assertEquals("Dane Smith", customer2.getName());

        // Read first customer from Database
        Customer customer3 = compositeReader.read();
        assertNotNull(customer3);
        assertEquals(3, customer3.getId());  // From Database
        assertEquals("Frank Martin", customer3.getName());

        // Read second customer from Database
        Customer customer4 = compositeReader.read();
        assertNotNull(customer4);
        assertEquals(4, customer4.getId());  // From Database
        assertEquals("Grace Lewis", customer4.getName());

    }
}
